@@ -54,8 +54,6 @@ class ClassDef(object):
                 amf3 = True
         self.amf3 = amf3
 
-        self._decoded_attrs = None # This value gets set by the decoder, don't mess with it.
-
     def getStaticAttrVals(self, obj):
         """Returns a list of values of attributes defined in self.static_attrs
 
@@ -118,12 +116,12 @@ class ExternizeableClassDef(ClassDef):
 
         The return value can be a string or a ByteArray.
         """
-
         raise ClassDefError("This method must be implemented by a sub-class.")
 
     def readByteString(self, obj, buf):
         """Returns an integer specifying the
-        position of the last decoded byte in the byte array.
+        0-indexed position of the last decoded
+        byte in the byte array.
 
         This method must be overridden in a sub-class.
 
@@ -132,7 +130,6 @@ class ExternizeableClassDef(ClassDef):
          * obj - object, The object that the byte string is being applied to.
          * buf - string in 2.4 and 2.5, ByteArray in 2.6+, The bytes to be read.
         """
-        
         raise ClassDefError("This method must be implemented by a sub-class.")
 
 class _ProxyClassDef(ExternizeableClassDef):
