@@ -1,10 +1,13 @@
 /* Things used by both the encoder and the decoder. */
 #include <Python.h>
 
-// Valid integer range
+// Valid AMF3 integer range
 #define MIN_INT -268435457
 #define MAX_INT 268435456
 
+// Valid AMF0 integer types
+#define MAX_USHORT 65535
+
 // Reference bit
 #define REFERENCE_BIT 0x01
 
@@ -17,6 +20,7 @@
 #define EXTERNIZEABLE 0x07
 
 // Type markers
+#define UNDEFINED_TYPE 0x00
 #define NULL_TYPE 0x01
 #define FALSE_TYPE 0x02
 #define TRUE_TYPE 0x03
