@@ -18,6 +18,7 @@ const int endian_test = 1;
 static PyObject *xml_dom_mod;
 static PyObject *amfast_mod;
 static PyObject *class_def_mod;
+static PyObject *as_types_mod;
 static PyObject *amfast_Error;
 static PyObject *amfast_EncodeError;
 static int big_endian; // Flag == 1 if architecture is big_endian, == 0 if not
@@ -41,7 +42,7 @@ typedef struct {
     int use_object_proxies; // Flag == 1 to encode dicts to ObjectProxy
     int use_references; // Flag == 1 to encode multiply occuring objects as references
     int use_legacy_xml; // Flag == 1 to encode XML as legacy XMLDocument instead of E4X
- 
+    int use_amf3; // Flag == 1 to encode packet messages in AMF3
 } EncoderContext;
 
 static EncoderContext* _create_encoder_context(size_t size, int amf3);
@@ -73,10 +74,7 @@ static int serialize_unicode(EncoderContext *context, PyObject *value);
 static int encode_unicode(EncoderContext *context, PyObject *value);
 static int serialize_string(EncoderContext *context, PyObject *value);
 static int encode_string(EncoderContext *context, PyObject *value);
-static int serialize_string_or_unicode(EncoderContext *context, PyObject *value);
-static int write_tuple(EncoderContext *context, PyObject *value);
-static int serialize_tuple(EncoderContext *context, PyObject *value);
-static int encode_tuple(EncoderContext *context, PyObject *value);
+static int serialize_object_as_string(EncoderContext *context, PyObject *value);
 static int write_list(EncoderContext *context, PyObject *value);
 static int serialize_list(EncoderContext *context, PyObject *value);
 static int encode_list(EncoderContext *context, PyObject *value);
@@ -96,8 +94,8 @@ static int serialize_class_def(EncoderContext *context, PyObject *value);
 static int encode_class_def(EncoderContext *context, PyObject *value);
 static int serialize_byte_array(EncoderContext *context, PyObject *value);
 static int encode_byte_array(EncoderContext *context, PyObject *value);
-
 static int check_xml(PyObject *value);
+static int check_byte_array(PyObject *value);
 
 // AMF0
 static int encode_bool_AMF0(EncoderContext *context, PyObject *value);
@@ -111,12 +109,11 @@ static int encode_string_AMF0(EncoderContext *context, PyObject *value, int allo
 static int write_unicode_AMF0(EncoderContext *context, PyObject *value);
 static int encode_unicode_AMF0(EncoderContext *context, PyObject *value, int allow_long);
 static int write_reference_AMF0(EncoderContext *context, PyObject *value);
-static int write_tuple_AMF0(EncoderContext *context, PyObject *value);
-static int write_list_AMF0(EncoderContext *context, PyObject *value);
+static int write_list_AMF0(EncoderContext *context, PyObject *value, int write_reference);
 static int write_dict_AMF0(EncoderContext *context, PyObject *value);
 static int _encode_dynamic_dict_AMF0(EncoderContext *context, PyObject *value);
-static int encode_string_or_unicode_AMF0(EncoderContext *context, PyObject *value, int allow_long);
-static int encode_date_AMF0(EncoderContext *context, PyObject *value);
+static int encode_object_as_string_AMF0(EncoderContext *context, PyObject *value, int allow_long);
+static int write_date_AMF0(EncoderContext *context, PyObject *value);
 static int encode_class_def_AMF0(EncoderContext *context, PyObject *value);
 static int _encode_object_AMF0(EncoderContext *context, PyObject *value);
 static int write_object_AMF0(EncoderContext *context, PyObject *value);
@@ -202,6 +199,7 @@ static EncoderContext* _create_encoder_context(size_t size, int amf3)
     }
 
     context->use_references = 1;
+    context->use_amf3 = 0;
     return context;
 }
 
@@ -217,6 +215,7 @@ static EncoderContext* _copy_encoder_context(EncoderContext *context, int amf3)
         return NULL;
 
     new_context->use_references = context->use_references;
+    new_context->use_amf3 = context->use_amf3;
     new_context->use_array_collections = context->use_array_collections;
     new_context->use_object_proxies = context->use_object_proxies;
 
@@ -372,8 +371,6 @@ static int _encode_double(EncoderContext *context, double value)
 static int encode_float(EncoderContext *context, PyObject *value)
 {
     double n = PyFloat_AsDouble(value);
-    if (n == -1.0)
-        return 0;
     return _encode_double(context, n);
 }
 
@@ -439,7 +436,7 @@ static int write_int(EncoderContext *context, PyObject *value)
         // Int is in valid AMF3 int range.
         if (_amf_write_byte(context, INT_TYPE) != 1)
             return 0;
-        return _encode_int((int)context, n);
+        return _encode_int(context, n);
     } else {
         // Int is too big, it must be encoded as a double
         if (_amf_write_byte(context, DOUBLE_TYPE) != 1)
@@ -529,20 +526,25 @@ static int encode_string(EncoderContext *context, PyObject *value)
 }
 
 /* Encode a PyString or a PyUnicode. */
-static int serialize_string_or_unicode(EncoderContext *context, PyObject *value)
+static int serialize_object_as_string(EncoderContext *context, PyObject *value)
 {
-    if (PyUnicode_Check(value)) {
-        return serialize_unicode(context, value);
-    } else if (PyString_Check(value)) {
+    if (PyString_Check(value)) {
         return serialize_string(context, value);
+    } else if (PyUnicode_Check(value)) {
+        return serialize_unicode(context, value);
     }
- 
-    PyErr_SetString(amfast_EncodeError, "Attempting to serialize non string/unicode as string.");
-    return 0;
+
+    PyObject *str_rep = PyObject_Str(value);
+    if (!str_rep)
+        return 0;
+
+    int return_value = serialize_string(context, str_rep);
+    Py_DECREF(str_rep);
+    return return_value;
 }
 
 /* Encode a PyString or a PyUnicode to AMF0. */
-static int encode_string_or_unicode_AMF0(EncoderContext *context, PyObject *value, int allow_long)
+static int encode_object_as_string_AMF0(EncoderContext *context, PyObject *value, int allow_long)
 {
     if (PyUnicode_Check(value)) {
         return encode_unicode_AMF0(context, value, allow_long);
@@ -550,8 +552,13 @@ static int encode_string_or_unicode_AMF0(EncoderContext *context, PyObject *valu
         return encode_string_AMF0(context, value, allow_long);
     }
 
-    PyErr_SetString(amfast_EncodeError, "Attempting to encode non string/unicode as string.");
-    return 0;
+    PyObject *str_rep = PyObject_Str(value);
+    if (!str_rep)
+        return 0;
+
+    int return_value = encode_string_AMF0(context, str_rep, allow_long);
+    Py_DECREF(str_rep);
+    return return_value;
 }
 
 /* Encode an ArrayCollection header. */
@@ -591,61 +598,7 @@ static int _encode_array_collection_header(EncoderContext *context)
     return _amf_write_byte(context, ARRAY_TYPE);
 }
 
-/* Write a PyTuple. */
-static int write_tuple(EncoderContext *context, PyObject *value)
-{
-    // Write type marker
-    if (context->use_array_collections == 1) {
-        if (!_amf_write_byte(context, OBJECT_TYPE))
-            return 0;
-    } else {
-        if (_amf_write_byte(context, ARRAY_TYPE) != 1)
-            return 0;
-    }
-
-    return serialize_tuple(context, value);
-}
-
-/* Serialize a PyTuple. */
-static int serialize_tuple(EncoderContext *context, PyObject *value)
-{
-    // Check for idx
-    int return_value = encode_reference(context, context->object_refs, value, 0);
-    if (return_value > -1)
-        return return_value;
-
-    // Write ArrayCollection header
-    if (context->use_array_collections == 1) {
-        if (!_encode_array_collection_header(context))
-            return 0;
-    }
-
-    return encode_tuple(context, value);
-}
-
-/* Encode a PyTuple. */
-static int encode_tuple(EncoderContext *context, PyObject *value)
-{
-    // Add size of tuple to header
-    Py_ssize_t value_len = PyTuple_GET_SIZE(value);
-    if (!_encode_int(context, ((int)value_len) << 1 | NULL_TYPE))
-        return 0;
-
-    // We're never writing associative array items
-    if (!_amf_write_byte(context, NULL_TYPE))
-        return 0;
-
-    // Encode each value in the tuple
-    int i;
-    for (i = 0; i < value_len; i++) {
-        if (!_encode(context, PyTuple_GET_ITEM(value, i)))
-            return 0;
-    }
-
-    return 1;
-}
-
-/* Writes a PyList. */
+/* Writes a PyList or PyTuple. */
 static int write_list(EncoderContext *context, PyObject *value)
 {
     // Write type marker
@@ -660,7 +613,7 @@ static int write_list(EncoderContext *context, PyObject *value)
     return serialize_list(context, value);
 }
 
-/* Serializes a PyList. */
+/* Serializes a PyList or PyTuple. */
 static int serialize_list(EncoderContext *context, PyObject *value)
 {
     // Check for idx
@@ -677,11 +630,14 @@ static int serialize_list(EncoderContext *context, PyObject *value)
     return encode_list(context, value);
 }
 
-/* Encode a PyList. */
+/* Encode a PyList or PyTuple. */
 static int encode_list(EncoderContext *context, PyObject *value)
 {
     // Add size of list to header
-    Py_ssize_t value_len = PyList_GET_SIZE(value);
+    Py_ssize_t value_len = PySequence_Size(value);
+    if (value_len < 0)
+        return 0;
+
     if (!_encode_int(context, ((int)value_len) << 1 | NULL_TYPE))
         return 0;
 
@@ -694,8 +650,10 @@ static int encode_list(EncoderContext *context, PyObject *value)
     for (i = 0; i < value_len; i++) {
         // Increment ref count in case 
         // list is modified by someone else.
-        PyObject *list_item = PyList_GET_ITEM(value, i);
-        Py_INCREF(list_item);
+        PyObject *list_item = PySequence_GetItem(value, i);
+        if (!list_item)
+            return 0;
+
         int return_value = _encode(context, list_item);
         Py_DECREF(list_item);
         if (!return_value)
@@ -779,10 +737,8 @@ static int _encode_dynamic_dict(EncoderContext *context, PyObject *value)
     Py_ssize_t idx = 0;
 
     while (PyDict_Next(value, &idx, &key, &val)) {
-        if (!serialize_string_or_unicode(context, key)) {
-            PyErr_SetString(amfast_EncodeError, "Non string/dict key. Only string and unicode dict keys can be encoded.");
+        if (!serialize_object_as_string(context, key))
             return 0;
-        }
 
         if (!_encode(context, val)) {
             return 0;
@@ -855,7 +811,7 @@ static int encode_reference(EncoderContext *context, ObjectContext *object_conte
 
     int idx = get_idx_from_ref(object_context, value);
     if (idx > -1) {
-       if (idx < MAX_INT) {// Max idx size (that's a lot of refs.)
+       if (idx < MAX_INT) {// Max reference count
            if (!_encode_int(context, (idx << (bit + 1)) | (0x00 + bit)))
                return 0;
            return 1;
@@ -884,13 +840,15 @@ static int write_reference_AMF0(EncoderContext *context, PyObject *value)
 
     int idx = get_idx_from_ref(context->object_refs, value);
     if (idx > -1) {
-        if (!_amf_write_byte(context, REF_AMF0))
-            return 0;
+        if (idx < MAX_USHORT) {
+            if (!_amf_write_byte(context, REF_AMF0))
+                return 0;
 
-        if (!_encode_ulong(context, (unsigned int)idx))
-            return 0;
+            if (!_encode_ushort(context, (unsigned short)idx))
+                return 0;
 
-        return 1;
+            return 1;
+        }
     }
 
     // Object is not indexed, add index
@@ -910,24 +868,36 @@ static int serialize_byte_array(EncoderContext *context, PyObject *value)
 
     // Length prefix
     Py_ssize_t value_len;
-    if (PyString_CheckExact(value)) {
-        value_len = PyString_GET_SIZE(value);
-    }
     #ifdef Py_BYTEARRAYOBJECT_H
     // ByteArray encoding is only available in 2.6+
-    else if (PyByteArray_Check(value)) {
+    if (PyByteArray_Check(value)) {
         value_len = PyByteArray_GET_SIZE(value);
     }
+    #else
+    PyObject *byte_string;
+    if (check_byte_array(value)) {
+        byte_string = PyObject_GetAttrString(value, "bytes");
+        if (!byte_string)
+            return 0;
+        value_len = PyString_GET_SIZE(byte_string);
+    }
     #endif
+
     else {
-        PyErr_SetString(amfast_EncodeError, "Cannot encode non ByteArray/String as byte array.");
+        PyErr_SetString(amfast_EncodeError, "Cannot encode non ByteArray/AsByteArray as byte array.");
         return 0;
     }
 
     if (!_encode_int(context, ((int)value_len) << 1 | REFERENCE_BIT))
         return 0;
 
+    #ifdef Py_BYTEARRAYOBJECT_H
     return encode_byte_array(context, value);
+    #else
+    return_value = encode_byte_array(context, byte_string);
+    Py_DECREF(byte_string);
+    return return_value;
+    #endif
 }
 
 /* Serializes a PyByteArray or a PyString. */
@@ -963,10 +933,11 @@ static int write_xml(EncoderContext *context, PyObject *value)
 {
     int byte_marker;
 
-    if (context->use_legacy_xml == 1)
+    if (context->use_legacy_xml == 1) {
         byte_marker = XML_DOC_TYPE;
-    else
+    } else {
         byte_marker = XML_TYPE;
+    }
 
     if (!_amf_write_byte(context, byte_marker))
         return 0;
@@ -974,6 +945,9 @@ static int write_xml(EncoderContext *context, PyObject *value)
     return serialize_xml(context, value);
 }
 
+
+
+
 /* Serializes an xml.dom.Document object. */
 static int serialize_xml(EncoderContext *context, PyObject *value)
 {
@@ -1010,6 +984,25 @@ static int check_xml(PyObject *value)
     return return_value;
 }
 
+/* Returns 1 if a PyObject is a amfast.class_def.as_types.AsByteArray object. */
+static int check_byte_array(PyObject *value)
+{
+    if (!as_types_mod) {
+        // Import amfast.class_def.as_types
+        as_types_mod = PyImport_ImportModule("amfast.class_def.as_types");
+        if (!as_types_mod)
+            return -1;
+    }
+
+    PyObject *class = PyObject_GetAttrString(as_types_mod, "AsByteArray");
+    if (!class)
+        return -1;
+
+    int return_value = PyObject_IsInstance(value, class);
+    Py_DECREF(class);
+    return return_value;
+}
+
 /* Serialize a Python object. */
 static int serialize_object(EncoderContext *context, PyObject *value)
 {
@@ -1178,7 +1171,7 @@ static int encode_class_def(EncoderContext *context, PyObject *value)
            return 0;
        }
        
-       int return_value = serialize_string_or_unicode(context, class_alias);
+       int return_value = serialize_object_as_string(context, class_alias);
        Py_DECREF(class_alias);
        return return_value;
     }
@@ -1211,7 +1204,7 @@ static int encode_class_def(EncoderContext *context, PyObject *value)
        return 0;
     }
 
-    int return_value = serialize_string_or_unicode(context, class_alias);
+    int return_value = serialize_object_as_string(context, class_alias);
     Py_DECREF(class_alias);
     if (!return_value) {
        Py_DECREF(static_attrs);
@@ -1227,7 +1220,7 @@ static int encode_class_def(EncoderContext *context, PyObject *value)
             return 0;
         }
         
-        int return_value = serialize_string_or_unicode(context, attr_name);
+        int return_value = serialize_object_as_string(context, attr_name);
         Py_DECREF(attr_name);
         if (!return_value)
             return 0;
@@ -1395,18 +1388,23 @@ static int encode_string_AMF0(EncoderContext *context, PyObject *value, int allo
 }
 
 /* Write a PyList to AMF0. */
-static int write_list_AMF0(EncoderContext *context, PyObject *value)
+static int write_list_AMF0(EncoderContext *context, PyObject *value, int write_reference)
 {
-    int return_value = write_reference_AMF0(context, value);
-    if (return_value == 0 || return_value == 1) {
-        return return_value;
+    if (write_reference) {
+        int return_value = write_reference_AMF0(context, value);
+        if (return_value == 0 || return_value == 1) {
+            return return_value;
+        }
     }
 
     // No reference
     if (!_amf_write_byte(context, ARRAY_AMF0))
         return 0;
 
-    Py_ssize_t array_len = PyList_GET_SIZE(value);
+    Py_ssize_t array_len = PySequence_Size(value);
+    if (array_len < 0)
+        return 0;
+
     if (!_encode_ulong(context, (unsigned int)array_len))
         return 0;
 
@@ -1414,8 +1412,10 @@ static int write_list_AMF0(EncoderContext *context, PyObject *value)
     for (i = 0; i < array_len; i++) {
         // Increment ref count in case 
         // list is modified by someone else.
-        PyObject *list_item = PyList_GET_ITEM(value, i);
-        Py_INCREF(list_item);
+        PyObject *list_item = PySequence_GetItem(value, (Py_ssize_t)i);
+        if (!list_item)
+            return 0;
+
         int return_value = _encode_AMF0(context, list_item);
         Py_DECREF(list_item);
         if (!return_value)
@@ -1425,31 +1425,6 @@ static int write_list_AMF0(EncoderContext *context, PyObject *value)
     return 1;
 }
 
-/* Write a PyTuple to AMF0. */
-static int write_tuple_AMF0(EncoderContext *context, PyObject *value)
-{
-    int return_value = write_reference_AMF0(context, value);
-    if (return_value == 0 || return_value == 1) {
-        return return_value;
-    }
-
-    // No reference
-    if (!_amf_write_byte(context, ARRAY_AMF0))
-        return 0;
-
-    Py_ssize_t array_len = PyTuple_GET_SIZE(value);
-    if (!_encode_ulong(context, (unsigned int)array_len))
-        return 0;
-
-    int i;
-    for (i = 0; i < array_len; i++) {
-        if (!_encode_AMF0(context, PyTuple_GET_ITEM(value, i)))
-            return 0;
-    }
-
-    return 1;
-}
-
 /* Write a PyDict to AMF0. */
 static int write_dict_AMF0(EncoderContext *context, PyObject *value)
 {
@@ -1473,10 +1448,8 @@ static int _encode_dynamic_dict_AMF0(EncoderContext *context, PyObject *value)
     Py_ssize_t idx = 0;
 
     while (PyDict_Next(value, &idx, &key, &val)) {
-        if (!encode_string_or_unicode_AMF0(context, key, 0)) {
-            PyErr_SetString(amfast_EncodeError, "Non string/dict key. Only string and unicode dict keys can be encoded.");
+        if (!encode_object_as_string_AMF0(context, key, 0))
             return 0;
-        }
 
         if (!_encode_AMF0(context, val)) {
             return 0;
@@ -1492,8 +1465,17 @@ static int _encode_dynamic_dict_AMF0(EncoderContext *context, PyObject *value)
 }
 
 /* Encode a Date object to AMF0 .*/
-static int encode_date_AMF0(EncoderContext *context, PyObject *value)
+static int write_date_AMF0(EncoderContext *context, PyObject *value)
 {
+    // Dates can use references
+    /*int wrote_ref = write_reference_AMF0(context, value);
+    if (wrote_ref == 0 || wrote_ref == 1) {
+        return wrote_ref;
+    }*/
+
+    if (!_amf_write_byte(context, DATE_AMF0))
+        return 0;
+ 
     // Call python function to get datetime
     if (!amfast_mod) {
         amfast_mod = PyImport_ImportModule("amfast");
@@ -1539,7 +1521,6 @@ static int write_xml_AMF0(EncoderContext *context, PyObject *value)
 /* Encode a Python object that doesn't have a C API in AMF0. */
 static int _encode_object_AMF0(EncoderContext *context, PyObject *value)
 {
-
     // Check for special object types
     int xml_value = check_xml(value);
     if (xml_value == -1) {
@@ -1619,7 +1600,7 @@ static int encode_class_def_AMF0(EncoderContext *context, PyObject *value)
     if (!alias)
         return 0;
 
-    int return_value = encode_string_or_unicode_AMF0(context, alias, 0);
+    int return_value = encode_object_as_string_AMF0(context, alias, 0);
     Py_DECREF(alias);
     return return_value;
 }
@@ -1938,7 +1919,7 @@ static int encode_packet_header_AMF0(EncoderContext *context, PyObject *value)
     if (!header_name)
         return 0;
 
-    int return_value = encode_string_or_unicode_AMF0(context, header_name, 0);
+    int return_value = encode_object_as_string_AMF0(context, header_name, 0);
     Py_DECREF(header_name);
     if (!return_value)
         return 0;
@@ -1982,7 +1963,7 @@ static int encode_packet_message_AMF0(EncoderContext *context, PyObject *value)
     if (!target)
         return 0;
 
-    int return_value = encode_string_or_unicode_AMF0(context, target, 0);
+    int return_value = encode_object_as_string_AMF0(context, target, 0);
     Py_DECREF(target);
     if (!return_value)
         return 0;
@@ -1991,18 +1972,36 @@ static int encode_packet_message_AMF0(EncoderContext *context, PyObject *value)
     if (!response)
         return 0;
 
-    return_value = encode_string_or_unicode_AMF0(context, response, 0);
-    Py_DECREF(response);
-    if (!return_value)
+    return_value = encode_object_as_string_AMF0(context, response, 0);
+    if (!return_value) {
+        Py_DECREF(response);
         return 0;
+    }
 
     PyObject *body = PyObject_GetAttrString(value, "value");
-    if (!body)
+    if (!body) {
+        Py_DECREF(response);
         return 0;
+    }
 
-    // Encode message value with a new context, so references are reset
-    EncoderContext *new_context = _copy_encoder_context(context, 0);
-    return_value = _encode_AMF0(new_context, body);
+    // Encode message body with a new context, so references are reset
+    EncoderContext *new_context = _copy_encoder_context(context, context->use_amf3);
+
+    if (PySequence_Size(response) > 0 && (PyList_Check(body) || PyTuple_Check(body))) {
+        // We're encoding a request,
+        // Don't count argument list 
+        // in reference count.
+        return_value = write_list_AMF0(new_context, body, 0);
+    } else {
+        if (context->use_amf3) {
+            if (!_amf_write_byte(new_context, AMF3_AMF0))
+                return 0;
+            return_value = _encode(new_context, body);
+        } else {
+            return_value = _encode_AMF0(new_context, body);
+        }
+    }
+    Py_DECREF(response);
     Py_DECREF(body);
     if (!return_value) {
         _destroy_encoder_context(new_context);
@@ -2030,6 +2029,17 @@ static int _encode_object(EncoderContext *context, PyObject *value)
         return write_xml(context, value);
     }
 
+    #ifndef Py_BYTEARRAYOBJECT_H
+    int byte_array_value = check_byte_array(value);
+    if (byte_array_value == -1) {
+        return 0;
+    } else if (byte_array_value == 1) {
+        if (!_amf_write_byte(context, BYTE_ARRAY_TYPE))
+            return 0;
+        return serialize_byte_array(context, value);
+    }
+    #endif
+
     // Generic object
     if (!_amf_write_byte(context, OBJECT_TYPE))
             return 0;
@@ -2063,16 +2073,12 @@ static int _encode_AMF0(EncoderContext *context, PyObject *value)
         return write_string_AMF0(context, value);
     } else if (PyUnicode_Check(value)) {
         return write_unicode_AMF0(context, value);
-    } else if (PyList_Check(value)) {
-        return write_list_AMF0(context, value);
-    } else if (PyTuple_Check(value)) {
-        return write_tuple_AMF0(context, value);
+    } else if (PyList_Check(value) || PyTuple_Check(value)) {
+        return write_list_AMF0(context, value, 1);
     } else if (PyDict_Check(value)) {
         return write_dict_AMF0(context, value);
     } else if (PyDateTime_Check(value) || PyDate_Check(value)) {
-        if (!_amf_write_byte(context, DATE_AMF0))
-            return 0;
-        return encode_date_AMF0(context, value);
+       return write_date_AMF0(context, value);
     } else {
         return _encode_object_AMF0(context, value);
     }
@@ -2104,10 +2110,8 @@ static int _encode(EncoderContext *context, PyObject *value)
         if (!_amf_write_byte(context, DOUBLE_TYPE))
             return 0;
         return encode_long(context, value);
-    } else if (PyList_Check(value)) {
+    } else if (PyList_Check(value) || PyTuple_Check(value)) {
         return write_list(context, value);
-    } else if (PyTuple_Check(value)) {
-        return write_tuple(context, value);
     } else if (PyDict_Check(value)) {
         if (!_amf_write_byte(context, OBJECT_TYPE))
             return 0;
@@ -2155,7 +2159,7 @@ static PyObject* encode(PyObject *self, PyObject *args, PyObject *kwargs)
         &use_legacy_xml, &amf3, &packet, &include_private, &class_def_mapper))
         return NULL;
 
-    EncoderContext *context = _create_encoder_context(1024, amf3);
+    EncoderContext *context = _create_encoder_context(1024, ((!packet) && amf3));
     if (!context) {
         Py_DECREF(class_def_mapper);
         Py_DECREF(include_private);
@@ -2167,6 +2171,7 @@ static PyObject* encode(PyObject *self, PyObject *args, PyObject *kwargs)
     context->use_object_proxies = use_object_proxies;
     context->use_references = use_references;
     context->use_legacy_xml = use_legacy_xml;
+    context->use_amf3 = amf3;
 
     if (include_private != Py_None) {
         context->include_private = include_private;
@@ -2207,7 +2212,7 @@ static PyObject* encode(PyObject *self, PyObject *args, PyObject *kwargs)
     int return_value;
     if (packet) {
         return_value = _encode_packet(context, value);
-    } else if (amf3) {
+    } else if (context->use_amf3) {
         return_value = _encode(context, value); 
     } else {
         return_value = _encode_AMF0(context, value);
