@@ -45,6 +45,9 @@ class Service(object):
     # Name of special service that handles command messages
     COMMAND_SERVICE = 'COMMAND_SERVICE'
 
+    # Name of special service that handles targets without a service prefix
+    DEFAULT_SERVICE = 'DEFAULT_SERVICE'
+
     SEPARATOR = '.' # Character used to separate service names and target names
 
     def __init__(self, name):
@@ -194,10 +197,11 @@ class Message(object):
         """Invoke an action on an AMF0 style message."""
         qualified_name = self.target.split(Service.SEPARATOR)
         if len(qualified_name) < 2:
-            raise RemotingError("Target name: '%s' is invalid. Target name must be in the form: 'service%starget'." % \
-                (self.target, Service.SEPARATOR))
-        target_name = qualified_name.pop()
-        service_name = Service.SEPARATOR.join(qualified_name)
+            target_name = self.target
+            service_name = Service.DEFAULT_SERVICE
+        else:
+            target_name = qualified_name.pop()
+            service_name = Service.SEPARATOR.join(qualified_name)
 
         target = service_mapper.getTargetByName(service_name, target_name)
         if target is None:
@@ -258,6 +262,12 @@ class Packet(object):
             messages = []
         self.messages = messages
 
+    def _getAmf3(self):
+        if self.version == self.FLASH_9:
+            return True
+        return False
+    is_amf3 = property(_getAmf3)
+
     def __str__(self):
         header_msg = "\n  ".join(["%s" % header for header in self.headers])
 
@@ -354,12 +364,12 @@ class Gateway(object):
             if response_packet is None:
                 return None
             else:
-                return self.encode_packet(response_packet)
+                return self.encode_packet(response_packet, request_packet.is_amf3)
         except Exception, exc:
             amfast.log_exc()
 
             if request_packet is not None:
-               return self.encode_packet(request_packet.fail(exc))
+               return self.encode_packet(request_packet.fail(exc), request_packet.is_amf3)
             else:
                 # There isn't much we can do if
                 # the request was not decoded correctly.
@@ -373,13 +383,13 @@ class Gateway(object):
         return decoder.decode(raw_packet, packet=True,
             class_def_mapper=self.class_def_mapper)
 
-    def encode_packet(self, packet):
+    def encode_packet(self, packet, amf3=False):
         raw_packet = encoder.encode(packet, packet=True, 
             class_def_mapper=self.class_def_mapper,
             use_array_collections=self.use_array_collections,
             use_object_proxies=self.use_object_proxies,
             use_references=self.use_references, use_legacy_xml=self.use_legacy_xml,
-            include_private=self.include_private)
+            include_private=self.include_private, amf3=amf3)
 
         if amfast.log_debug:
             amfast.logger.debug("<rawResponsePacket>%s</rawResponsePacket>" %
@@ -395,6 +405,7 @@ class ServiceMapper(object):
     packet_header_service - Service, a special service for AMF packet headers.
     message_header_service - Service, a special service for AMF message headers.
     command_service - Service, a special service for Flex CommandMessages.
+    default_service - Service, a special service for targets that don't have service specifiers.
     
     When an AMF packet or message is processed, the object checks
     the header_service Service for a target where target.name == header.name
@@ -421,6 +432,8 @@ class ServiceMapper(object):
         self.mapService(self.message_header_service)
         self.command_service = Service(Service.COMMAND_SERVICE)
         self.mapService(self.command_service)
+        self.default_service = Service(Service.DEFAULT_SERVICE)
+        self.mapService(self.default_service)
 
         self.command_service.setTarget(ExtCallableTarget(targets.ro_ping,
             messaging.CommandMessage.CLIENT_PING_OPERATION))
@@ -432,11 +445,6 @@ class ServiceMapper(object):
         ==========
          * service - Service, the service to map.
         """
-        if service.name in (Service.PACKET_HEADER_SERVICE,
-            Service.MESSAGE_HEADER_SERVICE, service.COMMAND_SERVICE):
-            if service.name in self._mapped_services:
-                raise RemotingError("'%s' name is reserved for internal use." % service.name)
-
         self._mapped_services[service.name] = service
 
     def unMapService(self, service):
@@ -446,10 +454,6 @@ class ServiceMapper(object):
         ==========
          * service - Service, the service to un-map.
         """
-        if service.name in (Service.PACKET_HEADER_SERVICE,
-            Service.MESSAGE_HEADER_SERVICE, Service.COMMAND_SERVICE):
-            raise RemotingError("'%s' name is reserved for internal use." % service.name)
-
         if service.name in self._mapped_services:
             del self._mapped_services[service.name]
 
